@@ -33,29 +33,30 @@
  *  POSSIBILITY OF SUCH DAMAGE.
  */ 
 
-/** \author Jia Pan */
-
+/** @author Jia Pan */
 
 #ifndef FCL_BROAD_PHASE_DYNAMIC_AABB_TREE_H
 #define FCL_BROAD_PHASE_DYNAMIC_AABB_TREE_H
 
-#include "fcl/broadphase/broadphase.h"
-#include "fcl/broadphase/hierarchy_tree.h"
-#include "fcl/BV/BV.h"
-#include "fcl/shape/geometric_shapes_utility.h"
 #include <unordered_map>
 #include <functional>
-#include <limits>
 
+#include "fcl/math/bv/utility.h"
+#include "fcl/geometry/shape/box.h"
+#include "fcl/geometry/shape/utility.h"
+#include "fcl/broadphase/broadphase_collision_manager.h"
+#include "fcl/broadphase/detail/hierarchy_tree.h"
 
 namespace fcl
 {
 
-class DynamicAABBTreeCollisionManager : public BroadPhaseCollisionManager
+template <typename S>
+class FCL_EXPORT DynamicAABBTreeCollisionManager : public BroadPhaseCollisionManager<S>
 {
 public:
-  typedef NodeBase<AABB> DynamicAABBNode;
-  typedef std::unordered_map<CollisionObject*, DynamicAABBNode*> DynamicAABBTable;
+
+  using DynamicAABBNode = detail::NodeBase<AABB<S>>;
+  using DynamicAABBTable = std::unordered_map<CollisionObject<S>*, DynamicAABBNode*> ;
 
   int max_tree_nonbalanced_level;
   int tree_incremental_balance_pass;
@@ -66,30 +67,16 @@ public:
   bool octree_as_geometry_collide;
   bool octree_as_geometry_distance;
 
-  
-  DynamicAABBTreeCollisionManager() : tree_topdown_balance_threshold(dtree.bu_threshold),
-                                      tree_topdown_level(dtree.topdown_level)
-  {
-    max_tree_nonbalanced_level = 10;
-    tree_incremental_balance_pass = 10;
-    tree_topdown_balance_threshold = 2;
-    tree_topdown_level = 0;
-    tree_init_level = 0;
-    setup_ = false;
-
-    // from experiment, this is the optimal setting
-    octree_as_geometry_collide = true;
-    octree_as_geometry_distance = false;
-  }
+  DynamicAABBTreeCollisionManager();
 
   /// @brief add objects to the manager
-  void registerObjects(const std::vector<CollisionObject*>& other_objs);
+  void registerObjects(const std::vector<CollisionObject<S>*>& other_objs);
   
   /// @brief add one object to the manager
-  void registerObject(CollisionObject* obj);
+  void registerObject(CollisionObject<S>* obj);
 
   /// @brief remove one object from the manager
-  void unregisterObject(CollisionObject* obj);
+  void unregisterObject(CollisionObject<S>* obj);
 
   /// @brief initialize the manager, related with the specific type of manager
   void setup();
@@ -98,69 +85,57 @@ public:
   void update();
 
   /// @brief update the manager by explicitly given the object updated
-  void update(CollisionObject* updated_obj);
+  void update(CollisionObject<S>* updated_obj);
 
   /// @brief update the manager by explicitly given the set of objects update
-  void update(const std::vector<CollisionObject*>& updated_objs);
+  void update(const std::vector<CollisionObject<S>*>& updated_objs);
 
   /// @brief clear the manager
-  void clear()
-  {
-    dtree.clear();
-    table.clear();
-  }
+  void clear();
 
   /// @brief return the objects managed by the manager
-  void getObjects(std::vector<CollisionObject*>& objs) const
-  {
-    objs.resize(this->size());
-    std::transform(table.begin(), table.end(), objs.begin(), std::bind(&DynamicAABBTable::value_type::first, std::placeholders::_1));
-  }
+  void getObjects(std::vector<CollisionObject<S>*>& objs) const;
 
   /// @brief perform collision test between one object and all the objects belonging to the manager
-  void collide(CollisionObject* obj, void* cdata, CollisionCallBack callback) const;
+  void collide(CollisionObject<S>* obj, void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance computation between one object and all the objects belonging to the manager
-  void distance(CollisionObject* obj, void* cdata, DistanceCallBack callback) const;
+  void distance(CollisionObject<S>* obj, void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief perform collision test for the objects belonging to the manager (i.e., N^2 self collision)
-  void collide(void* cdata, CollisionCallBack callback) const;
+  void collide(void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance test for the objects belonging to the manager (i.e., N^2 self distance)
-  void distance(void* cdata, DistanceCallBack callback) const;
+  void distance(void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief perform collision test with objects belonging to another manager
-  void collide(BroadPhaseCollisionManager* other_manager_, void* cdata, CollisionCallBack callback) const;
+  void collide(BroadPhaseCollisionManager<S>* other_manager_, void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance test with objects belonging to another manager
-  void distance(BroadPhaseCollisionManager* other_manager_, void* cdata, DistanceCallBack callback) const;
+  void distance(BroadPhaseCollisionManager<S>* other_manager_, void* cdata, DistanceCallBack<S> callback) const;
   
   /// @brief whether the manager is empty
-  bool empty() const
-  {
-    return dtree.empty();
-  }
+  bool empty() const;
   
   /// @brief the number of objects managed by the manager
-  size_t size() const
-  {
-    return dtree.size();
-  }
-
-  const HierarchyTree<AABB>& getTree() const { return dtree; }
+  size_t size() const;
 
+  const detail::HierarchyTree<AABB<S>>& getTree() const;
 
 private:
-  HierarchyTree<AABB> dtree;
-  std::unordered_map<CollisionObject*, DynamicAABBNode*> table;
+  detail::HierarchyTree<AABB<S>> dtree;
+  std::unordered_map<CollisionObject<S>*, DynamicAABBNode*> table;
 
   bool setup_;
 
-  void update_(CollisionObject* updated_obj);
+  void update_(CollisionObject<S>* updated_obj);
 };
 
+using DynamicAABBTreeCollisionManagerf = DynamicAABBTreeCollisionManager<float>;
+using DynamicAABBTreeCollisionManagerd = DynamicAABBTreeCollisionManager<double>;
 
+} // namespace fcl
 
-}
+#include "fcl/broadphase/broadphase_dynamic_AABB_tree-inl.h"
 
 #endif
