@@ -33,39 +33,33 @@
  *  POSSIBILITY OF SUCH DAMAGE.
  */ 
 
-/** \author Jia Pan */
+/** @author Jia Pan */
 
 #ifndef FCL_BROAD_PHASE_INTERVAL_TREE_H
 #define FCL_BROAD_PHASE_INTERVAL_TREE_H
 
-#include "fcl/broadphase/broadphase.h"
-#include "fcl/broadphase/interval_tree.h"
 #include <deque>
 #include <map>
+#include "fcl/broadphase/broadphase_collision_manager.h"
+#include "fcl/broadphase/detail/interval_tree.h"
 
 namespace fcl
 {
 
 /// @brief Collision manager based on interval tree
-class IntervalTreeCollisionManager : public BroadPhaseCollisionManager
+template <typename S>
+class FCL_EXPORT IntervalTreeCollisionManager : public BroadPhaseCollisionManager<S>
 {
 public:
-  IntervalTreeCollisionManager() : setup_(false)
-  {
-    for(int i = 0; i < 3; ++i)
-      interval_trees[i] = NULL;
-  }
+  IntervalTreeCollisionManager();
 
-  ~IntervalTreeCollisionManager()
-  {
-    clear();
-  }
+  ~IntervalTreeCollisionManager();
 
   /// @brief remove one object from the manager
-  void registerObject(CollisionObject* obj);
+  void registerObject(CollisionObject<S>* obj);
 
   /// @brief add one object to the manager
-  void unregisterObject(CollisionObject* obj);
+  void unregisterObject(CollisionObject<S>* obj);
 
   /// @brief initialize the manager, related with the specific type of manager
   void setup();
@@ -74,97 +68,110 @@ public:
   void update();
 
   /// @brief update the manager by explicitly given the object updated
-  void update(CollisionObject* updated_obj);
+  void update(CollisionObject<S>* updated_obj);
 
   /// @brief update the manager by explicitly given the set of objects update
-  void update(const std::vector<CollisionObject*>& updated_objs);
+  void update(const std::vector<CollisionObject<S>*>& updated_objs);
 
   /// @brief clear the manager
   void clear();
 
   /// @brief return the objects managed by the manager
-  void getObjects(std::vector<CollisionObject*>& objs) const;
+  void getObjects(std::vector<CollisionObject<S>*>& objs) const;
 
   /// @brief perform collision test between one object and all the objects belonging to the manager
-  void collide(CollisionObject* obj, void* cdata, CollisionCallBack callback) const;
+  void collide(CollisionObject<S>* obj, void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance computation between one object and all the objects belonging to the manager
-  void distance(CollisionObject* obj, void* cdata, DistanceCallBack callback) const;
+  void distance(CollisionObject<S>* obj, void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief perform collision test for the objects belonging to the manager (i.e., N^2 self collision)
-  void collide(void* cdata, CollisionCallBack callback) const;
+  void collide(void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance test for the objects belonging to the manager (i.e., N^2 self distance)
-  void distance(void* cdata, DistanceCallBack callback) const;
+  void distance(void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief perform collision test with objects belonging to another manager
-  void collide(BroadPhaseCollisionManager* other_manager, void* cdata, CollisionCallBack callback) const;
+  void collide(BroadPhaseCollisionManager<S>* other_manager, void* cdata, CollisionCallBack<S> callback) const;
 
   /// @brief perform distance test with objects belonging to another manager
-  void distance(BroadPhaseCollisionManager* other_manager, void* cdata, DistanceCallBack callback) const;
+  void distance(BroadPhaseCollisionManager<S>* other_manager, void* cdata, DistanceCallBack<S> callback) const;
 
   /// @brief whether the manager is empty
   bool empty() const;
   
   /// @brief the number of objects managed by the manager
-  inline size_t size() const { return endpoints[0].size() / 2; }
-
-
+  size_t size() const;
 
 protected:
 
   /// @brief SAP end point
-  struct EndPoint
-  {
-    /// @brief object related with the end point
-    CollisionObject* obj;
-
-    /// @brief end point value
-    FCL_REAL value;
-
-    /// @brief tag for whether it is a lower bound or higher bound of an interval, 0 for lo, and 1 for hi
-    char minmax;
-
-    bool operator<(const EndPoint &p) const
-    {
-        return value < p.value;
-    }
-  };
+  struct EndPoint;
 
   /// @brief Extention interval tree's interval to SAP interval, adding more information
-  struct SAPInterval : public SimpleInterval
-  {
-    CollisionObject* obj;
-    SAPInterval(double low_, double high_, CollisionObject* obj_) : SimpleInterval()
-    {
-      low = low_;
-      high = high_;
-      obj = obj_;
-    }
-  };
+  struct SAPInterval;
 
+  bool checkColl(
+      typename std::deque<detail::SimpleInterval<S>*>::const_iterator pos_start,
+      typename std::deque<detail::SimpleInterval<S>*>::const_iterator pos_end,
+      CollisionObject<S>* obj,
+      void* cdata,
+      CollisionCallBack<S> callback) const;
 
-  bool checkColl(std::deque<SimpleInterval*>::const_iterator pos_start, std::deque<SimpleInterval*>::const_iterator pos_end, CollisionObject* obj, void* cdata, CollisionCallBack callback) const;
+  bool checkDist(
+      typename std::deque<detail::SimpleInterval<S>*>::const_iterator pos_start,
+      typename std::deque<detail::SimpleInterval<S>*>::const_iterator pos_end,
+      CollisionObject<S>* obj,
+      void* cdata,
+      DistanceCallBack<S> callback,
+      S& min_dist) const;
 
-  bool checkDist(std::deque<SimpleInterval*>::const_iterator pos_start, std::deque<SimpleInterval*>::const_iterator pos_end, CollisionObject* obj, void* cdata, DistanceCallBack callback, FCL_REAL& min_dist) const;
+  bool collide_(CollisionObject<S>* obj, void* cdata, CollisionCallBack<S> callback) const;
 
-  bool collide_(CollisionObject* obj, void* cdata, CollisionCallBack callback) const;
-
-  bool distance_(CollisionObject* obj, void* cdata, DistanceCallBack callback, FCL_REAL& min_dist) const;
+  bool distance_(CollisionObject<S>* obj, void* cdata, DistanceCallBack<S> callback, S& min_dist) const;
 
   /// @brief vector stores all the end points
   std::vector<EndPoint> endpoints[3];
 
   /// @brief  interval tree manages the intervals
-  IntervalTree* interval_trees[3];
+  detail::IntervalTree<S>* interval_trees[3];
 
-  std::map<CollisionObject*, SAPInterval*> obj_interval_maps[3];
+  std::map<CollisionObject<S>*, SAPInterval*> obj_interval_maps[3];
 
   /// @brief tag for whether the interval tree is maintained suitably
   bool setup_;
 };
 
+using IntervalTreeCollisionManagerf = IntervalTreeCollisionManager<float>;
+using IntervalTreeCollisionManagerd = IntervalTreeCollisionManager<double>;
 
-}
+/// @brief SAP end point
+template <typename S>
+struct FCL_EXPORT IntervalTreeCollisionManager<S>::EndPoint
+{
+  /// @brief object related with the end point
+  CollisionObject<S>* obj;
+
+  /// @brief end point value
+  S value;
+
+  /// @brief tag for whether it is a lower bound or higher bound of an interval, 0 for lo, and 1 for hi
+  char minmax;
+
+  bool operator<(const EndPoint &p) const;
+};
+
+/// @brief Extention interval tree's interval to SAP interval, adding more information
+template <typename S>
+struct FCL_EXPORT IntervalTreeCollisionManager<S>::SAPInterval : public detail::SimpleInterval<S>
+{
+  CollisionObject<S>* obj;
+
+  SAPInterval(S low_, S high_, CollisionObject<S>* obj_);
+};
+
+} // namespace fcl
+
+#include "fcl/broadphase/broadphase_interval_tree-inl.h"
 
 #endif
