@@ -40,6 +40,7 @@
 
 #include "fcl/geometry/bvh/BVH_model.h"
 #include <new>
+#include <algorithm>
 
 namespace fcl
 {
@@ -92,7 +93,7 @@ BVHModel<BV>::BVHModel(const BVHModel<BV>& other)
   if(other.vertices)
   {
     vertices = new Vector3<S>[num_vertices];
-    memcpy(vertices, other.vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(other.vertices, other.vertices + num_vertices, vertices);
   }
   else
     vertices = nullptr;
@@ -100,7 +101,7 @@ BVHModel<BV>::BVHModel(const BVHModel<BV>& other)
   if(other.tri_indices)
   {
     tri_indices = new Triangle[num_tris];
-    memcpy(tri_indices, other.tri_indices, sizeof(Triangle) * num_tris);
+    std::copy(other.tri_indices, other.tri_indices + num_tris, tri_indices);
   }
   else
     tri_indices = nullptr;
@@ -108,7 +109,7 @@ BVHModel<BV>::BVHModel(const BVHModel<BV>& other)
   if(other.prev_vertices)
   {
     prev_vertices = new Vector3<S>[num_vertices];
-    memcpy(prev_vertices, other.prev_vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(other.prev_vertices, other.prev_vertices + num_vertices, prev_vertices);
   }
   else
     prev_vertices = nullptr;
@@ -129,7 +130,7 @@ BVHModel<BV>::BVHModel(const BVHModel<BV>& other)
     }
 
     primitive_indices = new unsigned int[num_primitives];
-    memcpy(primitive_indices, other.primitive_indices, sizeof(unsigned int) * num_primitives);
+    std::copy(other.primitive_indices, other.primitive_indices + num_primitives, primitive_indices);
   }
   else
     primitive_indices = nullptr;
@@ -138,7 +139,7 @@ BVHModel<BV>::BVHModel(const BVHModel<BV>& other)
   if(other.bvs)
   {
     bvs = new BVNode<BV>[num_bvs];
-    memcpy(bvs, other.bvs, sizeof(BVNode<BV>) * num_bvs);
+    std::copy(other.bvs, other.bvs + num_bvs, bvs);
   }
   else
     bvs = nullptr;
@@ -227,7 +228,7 @@ int BVHModel<BV>::beginModel(int num_tris_, int num_vertices_)
     tri_indices = new(std::nothrow) Triangle[num_tris_allocated];
     if(!tri_indices)
     {
-      std::cerr << "BVH Error! Out of memory for tri_indices array on BeginModel() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for tri_indices array on BeginModel() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
   }
@@ -235,13 +236,13 @@ int BVHModel<BV>::beginModel(int num_tris_, int num_vertices_)
   vertices = new Vector3<S>[num_vertices_allocated];
   if(!vertices)
   {
-    std::cerr << "BVH Error! Out of memory for vertices array on BeginModel() call!" << std::endl;
+    std::cerr << "BVH Error! Out of memory for vertices array on BeginModel() call!\n";
     return BVH_ERR_MODEL_OUT_OF_MEMORY;
   }
 
   if(build_state != BVH_BUILD_STATE_EMPTY)
   {
-    std::cerr << "BVH Warning! Call beginModel() on a BVHModel that is not empty. This model was cleared and previous triangles/vertices were lost." << std::endl;
+    std::cerr << "BVH Warning! Call beginModel() on a BVHModel that is not empty. This model was cleared and previous triangles/vertices were lost.\n";
     build_state = BVH_BUILD_STATE_EMPTY;
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
@@ -257,7 +258,7 @@ int BVHModel<BV>::addVertex(const Vector3<S>& p)
 {
   if(build_state != BVH_BUILD_STATE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call addVertex() in a wrong order. addVertex() was ignored. Must do a beginModel() to clear the model for addition of new vertices." << std::endl;
+    std::cerr << "BVH Warning! Call addVertex() in a wrong order. addVertex() was ignored. Must do a beginModel() to clear the model for addition of new vertices.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -266,11 +267,11 @@ int BVHModel<BV>::addVertex(const Vector3<S>& p)
     Vector3<S>* temp = new Vector3<S>[num_vertices_allocated * 2];
     if(!temp)
     {
-      std::cerr << "BVH Error! Out of memory for vertices array on addVertex() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for vertices array on addVertex() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
 
-    memcpy(temp, vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(vertices, vertices + num_vertices, temp);
     delete [] vertices;
     vertices = temp;
     num_vertices_allocated *= 2;
@@ -288,7 +289,7 @@ int BVHModel<BV>::addTriangle(const Vector3<S>& p1, const Vector3<S>& p2, const
 {
   if(build_state == BVH_BUILD_STATE_PROCESSED)
   {
-    std::cerr << "BVH Warning! Call addTriangle() in a wrong order. addTriangle() was ignored. Must do a beginModel() to clear the model for addition of new triangles." << std::endl;
+    std::cerr << "BVH Warning! Call addTriangle() in a wrong order. addTriangle() was ignored. Must do a beginModel() to clear the model for addition of new triangles.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -297,11 +298,11 @@ int BVHModel<BV>::addTriangle(const Vector3<S>& p1, const Vector3<S>& p2, const
     Vector3<S>* temp = new Vector3<S>[num_vertices_allocated * 2 + 2];
     if(!temp)
     {
-      std::cerr << "BVH Error! Out of memory for vertices array on addTriangle() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for vertices array on addTriangle() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
 
-    memcpy(temp, vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(vertices, vertices + num_vertices, temp);
     delete [] vertices;
     vertices = temp;
     num_vertices_allocated = num_vertices_allocated * 2 + 2;
@@ -325,11 +326,11 @@ int BVHModel<BV>::addTriangle(const Vector3<S>& p1, const Vector3<S>& p2, const
     Triangle* temp = new Triangle[num_tris_allocated * 2];
     if(!temp)
     {
-      std::cerr << "BVH Error! Out of memory for tri_indices array on addTriangle() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for tri_indices array on addTriangle() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
 
-    memcpy(temp, tri_indices, sizeof(Triangle) * num_tris);
+    std::copy(tri_indices, tri_indices + num_tris, temp);
     delete [] tri_indices;
     tri_indices = temp;
     num_tris_allocated *= 2;
@@ -347,7 +348,7 @@ int BVHModel<BV>::addSubModel(const std::vector<Vector3<S>>& ps)
 {
   if(build_state == BVH_BUILD_STATE_PROCESSED)
   {
-    std::cerr << "BVH Warning! Call addSubModel() in a wrong order. addSubModel() was ignored. Must do a beginModel() to clear the model for addition of new vertices." << std::endl;
+    std::cerr << "BVH Warning! Call addSubModel() in a wrong order. addSubModel() was ignored. Must do a beginModel() to clear the model for addition of new vertices.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -358,11 +359,11 @@ int BVHModel<BV>::addSubModel(const std::vector<Vector3<S>>& ps)
     Vector3<S>* temp = new Vector3<S>[num_vertices_allocated * 2 + num_vertices_to_add - 1];
     if(!temp)
     {
-      std::cerr << "BVH Error! Out of memory for vertices array on addSubModel() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for vertices array on addSubModel() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
 
-    memcpy(temp, vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(vertices, vertices + num_vertices, temp);
     delete [] vertices;
     vertices = temp;
     num_vertices_allocated = num_vertices_allocated * 2 + num_vertices_to_add - 1;
@@ -383,7 +384,7 @@ int BVHModel<BV>::addSubModel(const std::vector<Vector3<S>>& ps, const std::vect
 {
   if(build_state == BVH_BUILD_STATE_PROCESSED)
   {
-    std::cerr << "BVH Warning! Call addSubModel() in a wrong order. addSubModel() was ignored. Must do a beginModel() to clear the model for addition of new vertices." << std::endl;
+    std::cerr << "BVH Warning! Call addSubModel() in a wrong order. addSubModel() was ignored. Must do a beginModel() to clear the model for addition of new vertices.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -394,11 +395,11 @@ int BVHModel<BV>::addSubModel(const std::vector<Vector3<S>>& ps, const std::vect
     Vector3<S>* temp = new Vector3<S>[num_vertices_allocated * 2 + num_vertices_to_add - 1];
     if(!temp)
     {
-      std::cerr << "BVH Error! Out of memory for vertices array on addSubModel() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for vertices array on addSubModel() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
 
-    memcpy(temp, vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(vertices, vertices + num_vertices, temp);
     delete [] vertices;
     vertices = temp;
     num_vertices_allocated = num_vertices_allocated * 2 + num_vertices_to_add - 1;
@@ -424,11 +425,11 @@ int BVHModel<BV>::addSubModel(const std::vector<Vector3<S>>& ps, const std::vect
     Triangle* temp = new(std::nothrow) Triangle[num_tris_allocated * 2 + num_tris_to_add - 1];
     if(!temp)
     {
-      std::cerr << "BVH Error! Out of memory for tri_indices array on addSubModel() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for tri_indices array on addSubModel() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
 
-    memcpy(temp, tri_indices, sizeof(Triangle) * num_tris);
+    std::copy(tri_indices, tri_indices + num_tris, temp);
     delete [] tri_indices;
     tri_indices = temp;
     num_tris_allocated = num_tris_allocated * 2 + num_tris_to_add - 1;
@@ -450,13 +451,13 @@ int BVHModel<BV>::endModel()
 {
   if(build_state != BVH_BUILD_STATE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call endModel() in wrong order. endModel() was ignored." << std::endl;
+    std::cerr << "BVH Warning! Call endModel() in wrong order. endModel() was ignored.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
   if(num_tris == 0 && num_vertices == 0)
   {
-    std::cerr << "BVH Error! endModel() called on model with no triangles and vertices." << std::endl;
+    std::cerr << "BVH Error! endModel() called on model with no triangles and vertices.\n";
     return BVH_ERR_BUILD_EMPTY_MODEL;
   }
 
@@ -465,10 +466,10 @@ int BVHModel<BV>::endModel()
     Triangle* new_tris = new(std::nothrow) Triangle[num_tris];
     if(!new_tris)
     {
-      std::cerr << "BVH Error! Out of memory for tri_indices array in endModel() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for tri_indices array in endModel() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
-    memcpy(new_tris, tri_indices, sizeof(Triangle) * num_tris);
+    std::copy(tri_indices, tri_indices + num_tris, new_tris);
     delete [] tri_indices;
     tri_indices = new_tris;
     num_tris_allocated = num_tris;
@@ -479,10 +480,10 @@ int BVHModel<BV>::endModel()
     Vector3<S>* new_vertices = new Vector3<S>[num_vertices];
     if(!new_vertices)
     {
-      std::cerr << "BVH Error! Out of memory for vertices array in endModel() call!" << std::endl;
+      std::cerr << "BVH Error! Out of memory for vertices array in endModel() call!\n";
       return BVH_ERR_MODEL_OUT_OF_MEMORY;
     }
-    memcpy(new_vertices, vertices, sizeof(Vector3<S>) * num_vertices);
+    std::copy(vertices, vertices + num_vertices, new_vertices);
     delete [] vertices;
     vertices = new_vertices;
     num_vertices_allocated = num_vertices;
@@ -501,7 +502,7 @@ int BVHModel<BV>::endModel()
   primitive_indices = new(std::nothrow) unsigned int [num_bvs_to_be_allocated];
   if(!bvs || !primitive_indices)
   {
-    std::cerr << "BVH Error! Out of memory for BV array in endModel()!" << std::endl;
+    std::cerr << "BVH Error! Out of memory for BV array in endModel()!\n";
     return BVH_ERR_MODEL_OUT_OF_MEMORY;
   }
   num_bvs_allocated = num_bvs_to_be_allocated;
@@ -521,7 +522,7 @@ int BVHModel<BV>::beginReplaceModel()
 {
   if(build_state != BVH_BUILD_STATE_PROCESSED)
   {
-    std::cerr << "BVH Error! Call beginReplaceModel() on a BVHModel that has no previous frame." << std::endl;
+    std::cerr << "BVH Error! Call beginReplaceModel() on a BVHModel that has no previous frame.\n";
     return BVH_ERR_BUILD_EMPTY_PREVIOUS_FRAME;
   }
 
@@ -544,7 +545,7 @@ int BVHModel<BV>::replaceVertex(const Vector3<S>& p)
 {
   if(build_state != BVH_BUILD_STATE_REPLACE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call replaceVertex() in a wrong order. replaceVertex() was ignored. Must do a beginReplaceModel() for initialization." << std::endl;
+    std::cerr << "BVH Warning! Call replaceVertex() in a wrong order. replaceVertex() was ignored. Must do a beginReplaceModel() for initialization.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -560,7 +561,7 @@ int BVHModel<BV>::replaceTriangle(const Vector3<S>& p1, const Vector3<S>& p2, co
 {
   if(build_state != BVH_BUILD_STATE_REPLACE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call replaceTriangle() in a wrong order. replaceTriangle() was ignored. Must do a beginReplaceModel() for initialization." << std::endl;
+    std::cerr << "BVH Warning! Call replaceTriangle() in a wrong order. replaceTriangle() was ignored. Must do a beginReplaceModel() for initialization.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -576,7 +577,7 @@ int BVHModel<BV>::replaceSubModel(const std::vector<Vector3<S>>& ps)
 {
   if(build_state != BVH_BUILD_STATE_REPLACE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call replaceSubModel() in a wrong order. replaceSubModel() was ignored. Must do a beginReplaceModel() for initialization." << std::endl;
+    std::cerr << "BVH Warning! Call replaceSubModel() in a wrong order. replaceSubModel() was ignored. Must do a beginReplaceModel() for initialization.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -594,13 +595,13 @@ int BVHModel<BV>::endReplaceModel(bool refit, bool bottomup)
 {
   if(build_state != BVH_BUILD_STATE_REPLACE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call endReplaceModel() in a wrong order. endReplaceModel() was ignored. " << std::endl;
+    std::cerr << "BVH Warning! Call endReplaceModel() in a wrong order. endReplaceModel() was ignored. \n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
   if(num_vertex_updated != num_vertices)
   {
-    std::cerr << "BVH Error! The replaced model should have the same number of vertices as the old model." << std::endl;
+    std::cerr << "BVH Error! The replaced model should have the same number of vertices as the old model.\n";
     return BVH_ERR_INCORRECT_DATA;
   }
 
@@ -624,7 +625,7 @@ int BVHModel<BV>::beginUpdateModel()
 {
   if(build_state != BVH_BUILD_STATE_PROCESSED && build_state != BVH_BUILD_STATE_UPDATED)
   {
-    std::cerr << "BVH Error! Call beginUpdatemodel() on a BVHModel that has no previous frame." << std::endl;
+    std::cerr << "BVH Error! Call beginUpdatemodel() on a BVHModel that has no previous frame.\n";
     return BVH_ERR_BUILD_EMPTY_PREVIOUS_FRAME;
   }
 
@@ -653,7 +654,7 @@ int BVHModel<BV>::updateVertex(const Vector3<S>& p)
 {
   if(build_state != BVH_BUILD_STATE_UPDATE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call updateVertex() in a wrong order. updateVertex() was ignored. Must do a beginUpdateModel() for initialization." << std::endl;
+    std::cerr << "BVH Warning! Call updateVertex() in a wrong order. updateVertex() was ignored. Must do a beginUpdateModel() for initialization.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -669,7 +670,7 @@ int BVHModel<BV>::updateTriangle(const Vector3<S>& p1, const Vector3<S>& p2, con
 {
   if(build_state != BVH_BUILD_STATE_UPDATE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call updateTriangle() in a wrong order. updateTriangle() was ignored. Must do a beginUpdateModel() for initialization." << std::endl;
+    std::cerr << "BVH Warning! Call updateTriangle() in a wrong order. updateTriangle() was ignored. Must do a beginUpdateModel() for initialization.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -685,7 +686,7 @@ int BVHModel<BV>::updateSubModel(const std::vector<Vector3<S>>& ps)
 {
   if(build_state != BVH_BUILD_STATE_UPDATE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call updateSubModel() in a wrong order. updateSubModel() was ignored. Must do a beginUpdateModel() for initialization." << std::endl;
+    std::cerr << "BVH Warning! Call updateSubModel() in a wrong order. updateSubModel() was ignored. Must do a beginUpdateModel() for initialization.\n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
@@ -703,13 +704,13 @@ int BVHModel<BV>::endUpdateModel(bool refit, bool bottomup)
 {
   if(build_state != BVH_BUILD_STATE_UPDATE_BEGUN)
   {
-    std::cerr << "BVH Warning! Call endUpdateModel() in a wrong order. endUpdateModel() was ignored. " << std::endl;
+    std::cerr << "BVH Warning! Call endUpdateModel() in a wrong order. endUpdateModel() was ignored. \n";
     return BVH_ERR_BUILD_OUT_OF_SEQUENCE;
   }
 
   if(num_vertex_updated != num_vertices)
   {
-    std::cerr << "BVH Error! The updated model should have the same number of vertices as the old model." << std::endl;
+    std::cerr << "BVH Error! The updated model should have the same number of vertices as the old model.\n";
     return BVH_ERR_INCORRECT_DATA;
   }
 
@@ -743,10 +744,10 @@ int BVHModel<BV>::memUsage(int msg) const
   int total_mem = mem_bv_list + mem_tri_list + mem_vertex_list + sizeof(BVHModel<BV>);
   if(msg)
   {
-    std::cerr << "Total for model " << total_mem << " bytes." << std::endl;
-    std::cerr << "BVs: " << num_bvs << " allocated." << std::endl;
-    std::cerr << "Tris: " << num_tris << " allocated." << std::endl;
-    std::cerr << "Vertices: " << num_vertices << " allocated." << std::endl;
+    std::cerr << "Total for model " << total_mem << " bytes.\n";
+    std::cerr << "BVs: " << num_bvs << " allocated.\n";
+    std::cerr << "Tris: " << num_tris << " allocated.\n";
+    std::cerr << "Vertices: " << num_vertices << " allocated.\n";
   }
 
   return BVH_OK;
@@ -848,7 +849,7 @@ int BVHModel<BV>::buildTree()
     num_primitives = num_vertices;
     break;
   default:
-    std::cerr << "BVH Error: Model type not supported!" << std::endl;
+    std::cerr << "BVH Error: Model type not supported!\n";
     return BVH_ERR_UNSUPPORTED_FUNCTION;
   }
 
@@ -902,7 +903,7 @@ int BVHModel<BV>::recursiveBuildTree(int bv_id, int first_primitive, int num_pri
       }
       else
       {
-        std::cerr << "BVH Error: Model type not supported!" << std::endl;
+        std::cerr << "BVH Error: Model type not supported!\n";
         return BVH_ERR_UNSUPPORTED_FUNCTION;
       }
 
@@ -1011,7 +1012,7 @@ int BVHModel<BV>::recursiveRefitTree_bottomup(int bv_id)
     }
     else
     {
-      std::cerr << "BVH Error: Model type not supported!" << std::endl;
+      std::cerr << "BVH Error: Model type not supported!\n";
       return BVH_ERR_UNSUPPORTED_FUNCTION;
     }
   }
