@@ -177,27 +177,29 @@ bool GJKSolver_libccd<S>::shapeIntersect(
 
 // Shape intersect algorithms not using libccd
 //
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// |            | box | sphere | ellipsoid | capsule | cone | cylinder | plane | half-space | triangle |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | box        |  O  |   O    |           |         |      |          |   O   |      O     |          |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | sphere     |/////|   O    |           |    O    |      |    O     |   O   |      O     |    O     |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | ellipsoid  |/////|////////|           |         |      |          |   O   |      O     |   TODO   |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | capsule    |/////|////////|///////////|         |      |          |   O   |      O     |          |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | cone       |/////|////////|///////////|/////////|      |          |   O   |      O     |          |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | cylinder   |/////|////////|///////////|/////////|//////|          |   O   |      O     |          |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | plane      |/////|////////|///////////|/////////|//////|//////////|   O   |      O     |    O     |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | half-space |/////|////////|///////////|/////////|//////|//////////|///////|      O     |    O     |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
-// | triangle   |/////|////////|///////////|/////////|//////|//////////|///////|////////////|          |
-// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// |            | box | sphere | ellipsoid | capsule | cone | cylinder | plane | half-space | triangle |  convex  |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | box        |  O  |   O    |           |         |      |          |   O   |      O     |          |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | sphere     |/////|   O    |           |    O    |      |    O     |   O   |      O     |    O     |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | ellipsoid  |/////|////////|           |         |      |          |   O   |      O     |   TODO   |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | capsule    |/////|////////|///////////|         |      |          |   O   |      O     |          |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | cone       |/////|////////|///////////|/////////|      |          |   O   |      O     |          |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | cylinder   |/////|////////|///////////|/////////|//////|          |   O   |      O     |          |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | plane      |/////|////////|///////////|/////////|//////|//////////|   O   |      O     |    O     |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | half-space |/////|////////|///////////|/////////|//////|//////////|///////|      O     |    O     |    O     |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | triangle   |/////|////////|///////////|/////////|//////|//////////|///////|////////////|          |          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
+// | convex     |/////|////////|///////////|/////////|//////|//////////|///////|////////////|//////////|          |
+// +------------+-----+--------+-----------+---------+------+----------+-------+------------+----------+----------+
 
 #define FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT_REG(SHAPE1, SHAPE2, ALG)\
   template <typename S>\
@@ -255,6 +257,7 @@ FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Box, Halfspace, detail::boxHalfspaceInterse
 FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Capsule, Halfspace, detail::capsuleHalfspaceIntersect)
 FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Cylinder, Halfspace, detail::cylinderHalfspaceIntersect)
 FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Cone, Halfspace, detail::coneHalfspaceIntersect)
+FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Convex, Halfspace, detail::convexHalfspaceIntersect)
 
 FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Sphere, Plane, detail::spherePlaneIntersect)
 FCL_GJK_LIBCCD_SHAPE_SHAPE_INTERSECT(Ellipsoid, Plane, detail::ellipsoidPlaneIntersect)
