@@ -31,13 +31,19 @@
  *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  *  POSSIBILITY OF SUCH DAMAGE.
- */
+ */ 
 
-/** \author Jia Pan */
+/** @author Jia Pan */
 
-#include "fcl/broadphase/broadphase_spatialhash.h"
+#include "fcl/broadphase/broadphase_spatialhash-inl.h"
 
 namespace fcl
 {
 
-}
+template
+class SpatialHashingCollisionManager<
+    double,
+    detail::SimpleHashTable<
+        AABB<double>, CollisionObject<double>*, detail::SpatialHash<double>>>;
+
+} // namespace fcl
